## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup,results='hide'-----------------------------------------------------
library(EnrichDO)

## ----label=init,eval=T,echo=T,collapse=F,results='hide',cache=T---------------
demo.data=c(1636,351,102,2932,3077,348,4137,54209,5663,5328,23621,3416,3553)
doEnrich(interestGenes = demo.data, 
          test         = "hypergeomTest", 
          method       = "BH", 
          m            = 1, 
          maxGsize     = 5000,
          minGsize     = 5,
          traditional  = FALSE,
          delta        = 0.01, 
          result_do    = NULL,
          penalize     = T)

# [1] "Descending rights test"
# LEVEL: 13	1 nodes	72 genes to be scored
# LEVEL: 12	2 nodes	457 genes to be scored
# LEVEL: 11	3 nodes	907 genes to be scored
# LEVEL: 10	13 nodes	2279 genes to be scored
# LEVEL: 9	54 nodes	6504 genes to be scored
# LEVEL: 8	130 nodes	9483 genes to be scored
# LEVEL: 7	198 nodes	11209 genes to be scored
# LEVEL: 6	220 nodes	12574 genes to be scored
# LEVEL: 5	198 nodes	12936 genes to be scored
# LEVEL: 4	103 nodes	12824 genes to be scored
# LEVEL: 3	30 nodes	11683 genes to be scored
# LEVEL: 2	5 nodes	8032 genes to be scored
# LEVEL: 1	0 nodes	0 genes to be scored
# [1] "BH"
# [1] "hypergeomTest"

## ----eval=FALSE---------------------------------------------------------------
#  weight_result<-doEnrich(interestGenes = demo.data)

## ----eval=TRUE----------------------------------------------------------------
head(enrich)

## ----eval=TRUE----------------------------------------------------------------
head(doterms)

## ----eval=FALSE---------------------------------------------------------------
#  doEnrich(interestGenes= demo.data,
#           test         = "hypergeomTest",
#           method       = "holm",
#           m            = 1,
#           minGsize     = 5,
#           maxGsize     = 500,
#           delta        = 0.01,
#           penalize     = T)
#  

## ----eval=FALSE---------------------------------------------------------------
#  doEnrich(interestGenes = demo.data, penalize = F)
#  

## ----eval=FALSE---------------------------------------------------------------
#  doEnrich(demo.data , traditional = TRUE)
#  
#  # [1] "Traditional test"
#  # [1] "BH"
#  # [1] "hypergeomTest"

## ----eval=FALSE---------------------------------------------------------------
#  writeDoTerms(doterms,file = "doterms.txt")

## ----eval=FALSE---------------------------------------------------------------
#  writeResult(enrich,file = "result.txt",Q=1,P=1)

## ----fig.cap="bar plot",fig.align='center',fig.width=7,fig.height=5-----------
drawBarGraph(enrich,n=10,delta = 0.05)

## ----fig.cap="point plot",fig.align='center',fig.width=7,fig.height=5---------
drawPointGraph(enrich,n=10,delta = 0.05)

## ----fig.cap="tree plot",fig.align='center',fig.width=7,fig.height=5----------

drawGraphViz(enrich, n=10, numview=FALSE, pview=FALSE,labelfontsize = 17)


## ----fig.cap="heatmap",fig.align='center',fig.width=7,fig.height=5------------
drawHeatmap(interestGenes=demo.data,
enrich = enrich,
gene_n = 10,
fontsize_row = 8,
readable=T)

## ----eval=FALSE---------------------------------------------------------------
#  #Firstly, read the wrireResult output file,using the following two lines
#  #data<-read.delim(yourfile)
#  #doEnrich(result_do = data)
#  
#  #then, Use the drawing function you need
#  drawGraphViz(enrich)    #Tree diagram
#  drawPointGraph(enrich)  #Bubble diagram
#  drawBarGraph(enrich)    #Bar plot

